﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes the custom properties of a package. Custom properties are any properties that are not found in DismPackage Structure or DismFeature Structure.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824746.aspx"/>
		/// <![CDATA[typedef struct _DismCustomProperty
		/// {
		///		PCWSTR Name;
		///		PCWSTR Value;
		///		PCWSTR Path;
		///	} DismCustomProperty;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismCustomProperty_
		{
			/// <summary>
			/// The name of the custom property.
			/// </summary>
			public string Name;

			/// <summary>
			/// The value of the custom property.
			/// </summary>
			public string Value;

			/// <summary>
			/// The value of the custom property.
			/// </summary>
			public string Path;
		}
	}

	/// <summary>
	/// Represents the custom properties of a package. Custom properties are any properties that are not found in <see cref="DismPackage"/> or <see cref="DismFeature"/>.
	/// </summary>
	public class DismCustomProperty
	{
		private readonly DismApi.DismCustomProperty_ customProperty;

		/// <summary>
		/// Initializes a new instance of the DismCustomProperty class.
		/// </summary>
		/// <param name="customPropertyPtr">A native pointer to a <see cref="DismApi.DismCustomProperty_"/> struct.</param>
		internal DismCustomProperty(IntPtr customPropertyPtr)
			: this((DismApi.DismCustomProperty_)Marshal.PtrToStructure(customPropertyPtr, typeof(DismApi.DismCustomProperty_)))
		{

		}

		/// <summary>
		/// Initializes a new instance of the DismCustomProperty class.
		/// </summary>
		/// <param name="customProperty">A native <see cref="DismApi.DismCustomProperty_"/> struct that holds the data of the custom property</param>
		internal DismCustomProperty(DismApi.DismCustomProperty_ customProperty)
		{
			// Save a reference to the native struct
			this.customProperty = customProperty;
		}

		/// <summary>
		/// Gets the name of the custom property.
		/// </summary>
		public string Name
		{
			get
			{
				return this.customProperty.Name;
			}
		}

		/// <summary>
		/// Gets the value of the custom property.
		/// </summary>
		public string Value
		{
			get
			{
				return this.customProperty.Value;
			}
		}

		/// <summary>
		/// Gets the value of the custom property.
		/// </summary>
		public string Path
		{
			get
			{
				return this.customProperty.Path;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismCustomProperty"/> objects.
	/// </summary>
	public sealed class DismCustomPropertyCollection : DismCollection<DismCustomProperty>
	{
		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		internal DismCustomPropertyCollection()
			: base(new List<DismCustomProperty>())
		{

		}

		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		/// <param name="list"></param>
		internal DismCustomPropertyCollection(List<DismCustomProperty> list)
			: base(list)
		{

		}
	}
}
