﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes the architecture and hardware that the driver supports. The DismGetDriverInfo Function returns an object that includes an array of DismDriver structures. If you specify a DriverPath using the published name of the driver installed in the image, for example OEM1.inf, the array includes only the applicable hardware and architectures that are installed in the image. You can also specify a DriverPath using the source location of an .inf file on the technician computer. If you use the source location, the array includes all of the supported architectures and hardware that exist in the .inf file before it is installed to an architecture-specific image.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824796.aspx"/>
		/// <![CDATA[typedef struct _DismDriver
		///		{
		///		PCWSTR ManufacturerName;
		///		PCWSTR HardwareDescription;
		///		PCWSTR HardwareId;
		///		WORD Architecture;
		///		PCWSTR ServiceName;
		///		PCWSTR CompatibleIds;
		///		PCWSTR ExcludeIds;
		///		} DismDriverInfo;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismDriver_
		{
			/// <summary>
			/// The manufacturer name of the driver.
			/// </summary>
			public string ManufacturerName;

			/// <summary>
			/// A hardware description of the driver.
			/// </summary>
			public string HardwareDescription;

			/// <summary>
			/// The hardware ID of the driver.
			/// </summary>
			public string HardwareId;

			/// <summary>
			/// The architecture of the driver.
			/// </summary>
			public UInt16 Architecture;

			/// <summary>
			/// The service name of the driver.
			/// </summary>
			public string ServerName;

			/// <summary>
			/// The service name of the driver.
			/// </summary>
			public string CompatibleIds;

			/// <summary>
			/// The exclude IDs of the driver.
			/// </summary>
			public string ExcludeIds;
		}
	}

	/// <summary>
	/// Describes the architecture and hardware that the driver supports.
	/// </summary>
	public sealed class DismDriver
	{
		private readonly DismApi.DismDriver_ driver;

		/// <summary>
		/// Initializes a new instance of the DismDriver class.
		/// </summary>
		/// <param name="driverPtr">A native pointer to a DismDriver_ struct.</param>
		internal DismDriver(IntPtr driverPtr)
			: this((DismApi.DismDriver_)Marshal.PtrToStructure(driverPtr, typeof(DismApi.DismDriver_)))
		{

		}

		/// <summary>
		/// Initializes a new instance of the DismDriver class.
		/// </summary>
		/// <param name="driver">A native DismDriver_ struct to copy data from.</param>
		internal DismDriver(DismApi.DismDriver_ driver)
		{
			// Copy data from the struct
			this.driver = driver;
		}

		/// <summary>
		/// Gets the manufacturer name of the driver.
		/// </summary>
		public string ManufacturerName
		{
			get
			{
				return this.driver.ManufacturerName;
			}
		}

		/// <summary>
		/// Gets the hardware description of the driver.
		/// </summary>
		public string HardwareDescription
		{
			get
			{
				return this.driver.HardwareDescription;
			}
		}

		/// <summary>
		/// Gets the hardware ID of the driver.
		/// </summary>
		public string HardwareId
		{
			get
			{
				return this.driver.HardwareId;
			}
		}

		/// <summary>
		/// Gets the architecture of the driver.
		/// </summary>
		public DismProcessorArchitecture Architecture
		{
			get
			{
				return (DismProcessorArchitecture)this.driver.Architecture;
			}
		}

		/// <summary>
		/// Gets the service name of the driver.
		/// </summary>
		public string ServerName
		{
			get
			{
				return this.driver.ServerName;
			}
		}

		/// <summary>
		/// Gets the service name of the driver.
		/// </summary>
		public string CompatibleIds
		{
			get
			{
				return this.driver.CompatibleIds;
			}
		}

		/// <summary>
		/// Gets the exclude IDs of the driver.
		/// </summary>
		public string ExcludeIds
		{
			get
			{
				return this.driver.ExcludeIds;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismDriver"/> objects.
	/// </summary>
	public sealed class DismDriverCollection : DismCollection<DismDriver>
	{
		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		internal DismDriverCollection()
			: base(new List<DismDriver>())
		{

		}

		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		/// <param name="list"></param>
		internal DismDriverCollection(List<DismDriver> list)
			: base(list)
		{

		}
	}
}
