﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Contains basic information for the driver that is associated with the .inf file.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824776.aspx"/>
		/// <![CDATA[typedef struct DismDriverPackage
		///	{ 
		///		PCWSTR PublishedName;
		///		PCWSTR OriginalFileName;
		///		BOOL InBox;
		///		PCWSTR CatalogFile;
		///		PCWSTR ClassName;
		///		PCWSTR ClassGuid;
		///		PCWSTR ClassDescription;
		///		BOOL BootCritical;
		///		DismDriverSignature DriverSignature;
		///		PCWSTR ProviderName;
		///		SYSTEMTIME Date;
		///		UINT MajorVersion;
		///		UINT MinorVersion;
		///		UINT Build;
		///		UINT Revision
		///	} DismDriver;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismDriverPackage_
		{
			/// <summary>
			/// The published driver name.
			/// </summary>
			public string PublishedName;

			/// <summary>
			/// The original file name of the driver.
			/// </summary>
			public string OriginalFileName;

			/// <summary>
			/// TRUE if the driver is included on the Windows distribution media and automatically installed as part of Windows®, otherwise FALSE.
			/// </summary>
			public bool InBox;

			/// <summary>
			/// The catalog file for the driver.
			/// </summary>
			public string CatalogFile;

			/// <summary>
			/// The class name of the driver.
			/// </summary>
			public string ClassName;

			/// <summary>
			/// The class GUID of the driver.
			/// </summary>
			public string ClassGuid;

			/// <summary>
			/// The class description of the driver.
			/// </summary>
			public string ClassDescription;

			/// <summary>
			/// TRUE if the driver is boot-critical, otherwise FALSE.
			/// </summary>
			public bool BootCritical;

			/// <summary>
			/// The driver signature status.
			/// </summary>
			public DismDriverSignature DriverSignature;

			/// <summary>
			/// The provider of the driver.
			/// </summary>
			public string ProviderName;

			/// <summary>
			/// The manufacturer's build date of the driver.
			/// </summary>
			public SYSTEMTIME Date;

			/// <summary>
			/// The major version number of the driver.
			/// </summary>
			public UInt32 MajorVersion;

			/// <summary>
			/// The minor version number of the driver.
			/// </summary>
			public UInt32 MinorVersion;

			/// <summary>
			/// The build number of the driver.
			/// </summary>
			public UInt32 Build;

			/// <summary>
			/// The revision number of the driver.
			/// </summary>
			public UInt32 Revision;
		}
	}

	/// <summary>
	/// Represents basic information for the driver that is associated with the .inf file.
	/// </summary>
	public sealed class DismDriverPackage
	{
		private readonly DismApi.DismDriverPackage_ driverPackage;

		/// <summary>
		/// Initializes a new instance of the DismDriverPackage class.
		/// </summary>
		/// <param name="driverPackagePtr">A pointer to a native DismDriverPackage_ struct.</param>
		internal DismDriverPackage(IntPtr driverPackagePtr)
			: this((DismApi.DismDriverPackage_)Marshal.PtrToStructure(driverPackagePtr, typeof(DismApi.DismDriverPackage_)))
		{

		}

		/// <summary>
		/// Initializes a new instance of the DismDriverPackage class.
		/// </summary>
		/// <param name="driverPackage">A native DismDriverPackage_ struct.</param>
		internal DismDriverPackage(DismApi.DismDriverPackage_ driverPackage)
		{
			this.driverPackage = driverPackage;

			// Copy data from the struct
			this.Version = new Version((int)driverPackage.MajorVersion, (int)driverPackage.MinorVersion, (int)driverPackage.Build, (int)driverPackage.Revision);
		}

		/// <summary>
		/// Gets the published driver name.
		/// </summary>
		public string PublishedName
		{
			get
			{
				return this.driverPackage.PublishedName;
			}
		}

		/// <summary>
		/// Gets the original file name of the driver.
		/// </summary>
		public string OriginalFileName
		{
			get
			{
				return this.driverPackage.OriginalFileName;
			}
		}

		/// <summary>
		/// Gets a value indicating if the driver is included on the Windows distribution media and automatically installed as part of Windows®.
		/// </summary>
		public bool InBox
		{
			get
			{
				return this.driverPackage.InBox;
			}
		}

		/// <summary>
		/// Gets the catalog file for the driver.
		/// </summary>
		public string CatalogFile
		{
			get
			{
				return this.driverPackage.CatalogFile;
			}
		}

		/// <summary>
		/// Gets the class name of the driver.
		/// </summary>
		public string ClassName
		{
			get
			{
				return this.driverPackage.ClassName;
			}
		}

		/// <summary>
		/// Gets the class GUID of the driver.
		/// </summary>
		public string ClassGuid
		{
			get
			{
				return this.driverPackage.ClassGuid;
			}
		}

		/// <summary>
		/// Gets the class description of the driver.
		/// </summary>
		public string ClassDescription
		{
			get
			{
				return this.driverPackage.ClassDescription;
			}
		}

		/// <summary>
		/// Gets a value indicating if the driver is boot-critical.
		/// </summary>
		public bool BootCritical
		{
			get
			{
				return this.driverPackage.BootCritical;
			}
		}

		/// <summary>
		/// Gets the driver signature status.
		/// </summary>
		public DismDriverSignature DriverSignature
		{
			get
			{
				return this.driverPackage.DriverSignature;
			}
		}

		/// <summary>
		/// Gets the provider of the driver.
		/// </summary>
		public string ProviderName
		{
			get
			{
				return this.driverPackage.ProviderName;
			}
		}

		/// <summary>
		/// Gets the manufacturer's build date of the driver.
		/// </summary>
		public DateTime Date
		{
			get
			{
				return this.driverPackage.Date;
			}
		}

		/// <summary>
		/// Gets the major version number of the driver.
		/// </summary>
		public Version Version
		{
			get;
			private set;
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismDriverPackage"/> objects.
	/// </summary>
	public sealed class DismDriverPackageCollection : DismCollection<DismDriverPackage>
	{
		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		internal DismDriverPackageCollection()
			: base(new List<DismDriverPackage>())
		{

		}

		/// <summary>
		/// Initializes a new instance of the TextWriter class.
		/// </summary>
		/// <param name="list"></param>
		internal DismDriverPackageCollection(List<DismDriverPackage> list)
			: base(list)
		{

		}
	}
}
