﻿using System;
using System.Runtime.InteropServices;
using Microsoft.Dism.Properties;

namespace Microsoft.Dism
{
	/// <summary>
	/// Represents an exception in the DismApi.
	/// </summary>
	[Serializable]
	public class DismException : Exception
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="DismException"/> class with a specified error message and a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="message">The error message that explains the reason for the exception.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or a null reference (Nothing in Visual Basic) if no inner exception is specified.</param>
		internal DismException(string message, Exception innerException)
			: base(message, innerException)
		{

		}

		/// <summary>
		/// Initializes a new instance of the <see cref="DismException"/> class with a specified error that is the cause of this exception.
		/// </summary>
		/// <param name="message">The error message that explains the reason for the exception.</param>
		internal DismException(string message)
			: base(message)
		{

		}

		/// <summary>
		/// Gets a <see cref="DismException"/> or <see cref="Exception"/> for the specified error code.
		/// </summary>
		/// <param name="errorCode">The error code to get an exception for.</param>
		/// <returns>A <see cref="DismException"/> or <see cref="Exception"/> that represents the error code.</returns>
		internal static Exception GetDismExceptionForHR(int errorCode)
		{
			// Look for known error codes
			switch ((uint)errorCode)
			{
				case 0x800704D3 :// ERROR_REQUEST_ABORTED 
					return new OperationCanceledException();
				case DismApi.DISMAPI_E_DISMAPI_NOT_INITIALIZED :
					// User has not called DismApi.Initialize()
					return new DismNotInitializedException();
				case DismApi.DISMAPI_E_OPEN_SESSION_HANDLES :
					// User has not called CloseSession() on open sessions
					return new DismOpenSessionsException();
			}

			// Attempt to get an error message from the DismApi
			string lastError = DismApi.GetLastErrorMessage();

			// See if the result is not null
			if (String.IsNullOrEmpty(lastError) == false)
			{
				// Return a DismException object
				return new DismException(lastError.Trim(), Marshal.GetExceptionForHR(errorCode));
			}
			else
			{
				// Return an Exception for the HResult
				return Marshal.GetExceptionForHR(errorCode);
			}
		}
	}

	/// <summary>
	/// The exception that is thrown when an attempt to use the DismApi occurs without first calling <see cref="DismApi.Initialize(DismLogLevel)"/>.
	/// </summary>
	[Serializable]
	public sealed class DismNotInitializedException : DismException
	{
		internal DismNotInitializedException()
			: this(null)
		{
		}

		internal DismNotInitializedException(Exception innerException)
			: base(Resources.DismExceptionMessageNotInitialized, innerException)
		{

		}
	}

	/// <summary>
	/// The exception that is thrown when an attempt to shutdown the Dism API occurs while there are open sessions.
	/// </summary>
	[Serializable]
	public sealed class DismOpenSessionsException : DismException
	{
		internal DismOpenSessionsException()
			: this(null)
		{
		}

		internal DismOpenSessionsException(Exception innerException)
			: base(Resources.DismExceptionMessageOpenSessions, innerException)
		{

		}
	}
}