﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes basic information about a feature, such as the feature name and feature state.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824780.aspx"/>
		/// <![CDATA[typedef struct _DismFeature
		///	{
		///		PCWSTR FeatureName;
		///		DismPackageFeatureState State;
		///	} DismFeature;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismFeature_
		{
			/// <summary>
			/// The name of the feature.
			/// </summary>
			public string FeatureName;

			/// <summary>
			/// A valid DismPackageFeatureState Enumeration value such as DismStateInstalled.
			/// </summary>
			public DismPackageFeatureState State;
		}
	}

	/// <summary>
	/// Describes basic information about a feature, such as the feature name and feature state.
	/// </summary>
	public sealed class DismFeature
	{
		private readonly DismApi.DismFeature_ feature;
		

		/// <summary>
		/// Initializes a new instance of the DismFeature class.
		/// </summary>
		/// <param name="featurePtr">A native pointer to a DismFeature_ struct.</param>
		internal DismFeature(IntPtr featurePtr)
			: this((DismApi.DismFeature_)Marshal.PtrToStructure(featurePtr, typeof(DismApi.DismFeature_)))
		{
		}
		
		/// <summary>
		/// Initializes a new instance of the DismFeature class.
		/// </summary>
		/// <param name="feature">A native DismFeature_ struct to copy data from.</param>
		internal DismFeature(DismApi.DismFeature_ feature)
		{
			this.feature = feature;
		}

		/// <summary>
		/// Gets the name of the feature.
		/// </summary>
		public string FeatureName
		{
			get
			{
				return this.feature.FeatureName;
			}
		}

		/// <summary>
		/// Gets the state of the feature.
		/// </summary>
		public DismPackageFeatureState State
		{
			get
			{
				return this.feature.State;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismFeature"/> objects.
	/// </summary>
	public sealed class DismFeatureCollection : DismCollection<DismFeature>
	{
		/// <summary>
		/// Initializes a new instance of the DismFeatureCollection class.
		/// </summary>
		internal DismFeatureCollection()
			: base(new List<DismFeature>())
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismFeatureCollection class.
		/// </summary>
		/// <param name="list">An existing list of DismFeature objects to expose as a read-only collection.</param>
		internal DismFeatureCollection(IList<DismFeature> list)
			: base(list)
		{
		}
	}
}