﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes advanced feature information, such as installed state and whether a restart is required after installation.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824793.aspx"/>
		/// <![CDATA[typedef struct _DismFeatureInfo
		/// {
		///     PCWSTR FeatureName;
		///     DismPackageFeatureState FeatureState;
		///     PCWSTR DisplayName;
		///     PCWSTR Description;
		///     DismRestartType RestartRequired;
		///     DismCustomProperty* CustomProperty;
		///     UINT CustomPropertyCount;
		/// } DismFeatureInfo;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismFeatureInfo_
		{
			/// <summary>
			/// The name of the feature.
			/// </summary>
			public string FeatureName;

			/// <summary>
			/// A valid DismPackageFeatureState Enumeration value such as DismStateInstalled or 7.
			/// </summary>
			public DismPackageFeatureState FeatureState;

			/// <summary>
			/// The display name of the feature. This is not always unique across all features.
			/// </summary>
			public string DisplayName;

			/// <summary>
			/// The description of the feature.
			/// </summary>
			public string Description;

			/// <summary>
			/// A DismRestartType Enumeration value such as DismRestartPossible.
			/// </summary>
			public DismRestartType RestartRequired;

			/// <summary>
			/// An array of DismCustomProperty Structure.m
			/// </summary>
			public IntPtr CustomProperty;

			/// <summary>
			/// The number of elements in the CustomProperty array.
			/// </summary>
			public UInt32 CustomPropertyCount;
		}
	}

	/// <summary>
	/// Represents advanced feature information, such as installed state and whether a restart is required after installation.
	/// </summary>
	public sealed class DismFeatureInfo
	{
		private readonly DismApi.DismFeatureInfo_ featureInfo;
		private readonly DismCustomPropertyCollection customProperties = new DismCustomPropertyCollection();

		/// <summary>
		/// Initializes a new instance of the DismFeatureInfo class.
		/// </summary>
		/// <param name="featureInfoPtr">A pointer to a <see cref="DismApi.DismFeatureInfo_"/> struct.</param>
		internal DismFeatureInfo(IntPtr featureInfoPtr)
			: this((DismApi.DismFeatureInfo_)Marshal.PtrToStructure(featureInfoPtr, typeof(DismApi.DismFeatureInfo_)))
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismFeatureInfo class.
		/// </summary>
		/// <param name="featureInfo">A <see cref="DismApi.DismFeatureInfo_"/> struct from the native DismApi.</param>
		internal DismFeatureInfo(DismApi.DismFeatureInfo_ featureInfo)
		{
			// Save a reference to the native struct
			this.featureInfo = featureInfo;

			// See if there are any custom properties to load
			if (this.featureInfo.CustomPropertyCount > 0 && this.featureInfo.CustomProperty != IntPtr.Zero)
			{
				// Load the custom properties from the array of pointers
				DismApi.GetIntPtrList(this.featureInfo.CustomProperty, (int)this.featureInfo.CustomPropertyCount, typeof(DismApi.DismCustomProperty_)).ForEach(ptr => this.customProperties.Add(new DismCustomProperty(ptr)));
			}
		}

		/// <summary>
		/// Gets the name of the feature.
		/// </summary>
		public string FeatureName
		{
			get
			{
				return this.featureInfo.FeatureName;
			}
		}

		/// <summary>
		/// Gets the state of the feature.
		/// </summary>
		public DismPackageFeatureState FeatureState
		{
			get
			{
				return this.featureInfo.FeatureState;
			}
		}

		/// <summary>
		/// Gets the display name of the feature. This is not always unique across all features.
		/// </summary>
		public string DisplayName
		{
			get
			{
				return this.featureInfo.DisplayName;
			}
		}

		/// <summary>
		/// Gets the description of the feature.
		/// </summary>
		public string Description
		{
			get
			{
				return this.featureInfo.Description;
			}
		}

		/// <summary>
		/// Gets a value indicating if a restart is required when installing or uninstalling the feature.
		/// </summary>
		public DismRestartType RestartRequired
		{
			get
			{
				return this.featureInfo.RestartRequired;
			}
		}

		/// <summary>
		/// Gets a list of custom properties associated with the feature.
		/// </summary>
		public DismCustomPropertyCollection CustomProperties
		{
			get
			{
				return this.customProperties;
			}
		}
	}
}
