﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes the metadata of an image.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824797.aspx" />
		/// <![CDATA[typedef struct _DismImageInfo
		/// {
		///     DismImageType ImageType;
		///     UINT ImageIndex;
		///     PCWSTR ImageName;
		///     PCWSTR ImageDescription;
		///     UINT64 ImageSize;
		///     UINT Architecture;
		///     PCWSTR ProductName;
		///     PCWSTR EditionId;
		///     PCWSTR InstallationType;
		///     PCWSTR Hal;
		///     PCWSTR ProductType;
		///     PCWSTR ProductSuite;
		///     UINT MajorVersion;
		///     UINT MinorVersion;
		///     UINT Build;
		///     UINT SpBuild;
		///     UINT SpLevel;
		///     DismImageBootable Bootable;
		///     PCWSTR SystemRoot;
		///     DismLanguage* Language;
		///     UINT LanguageCount;
		///     UINT DefaultLanguage Index;
		///     VOID* CustomizedInfo;
		/// } DismImageInfo;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismImageInfo_
		{
			/// <summary>
			/// A DismImageType Enumeration value such as DismImageTypeWim.
			/// </summary>
			public DismImageType ImageType;

			/// <summary>
			/// The index number, starting at 1, of the image.
			/// </summary>
			public uint ImageIndex;

			/// <summary>
			/// The name of the image.
			/// </summary>
			public string ImageName;

			/// <summary>
			/// A description of the image.
			/// </summary>
			public string ImageDescription;

			/// <summary>
			/// The size of the image in bytes.
			/// </summary>
			public ulong ImageSize;

			/// <summary>
			/// The architecture of the image.
			/// </summary>
			public DismProcessorArchitecture Architecture;

			/// <summary>
			/// The name of the product, for example, "Microsoft Windows Operating System".
			/// </summary>
			public string ProductName;

			/// <summary>
			/// The edition of the product, for example, "Ultimate".
			/// </summary>
			public string EditionId;

			/// <summary>
			/// A string identifying whether the installation is a Client or Server type.
			/// </summary>
			public string InstallationType;

			/// <summary>
			/// The hardware abstraction layer (HAL) type of the operating system.
			/// </summary>
			public string Hal;

			/// <summary>
			/// The product type, for example, "WinNT".
			/// </summary>
			public string ProductType;

			/// <summary>
			/// The product suite, for example, "Terminal Server".
			/// </summary>
			public string ProductSuite;

			/// <summary>
			/// The major version of the operating system.
			/// </summary>
			public uint MajorVersion;

			/// <summary>
			/// The minor version of the operating system.
			/// </summary>
			public uint MinorVersion;

			/// <summary>
			/// The build number, for example, "7600".
			/// </summary>
			public uint Build;

			/// <summary>
			/// The service pack build.
			/// </summary>
			public uint SpBuild;

			/// <summary>
			/// The service pack number.
			/// </summary>
			public uint SpLevel;

			/// <summary>
			/// A DismImageBootable Enumeration value such as DismImageBootableYes.
			/// </summary>
			public DismImageBootable Bootable;

			/// <summary>
			/// The Windows directory.
			/// </summary>
			public string SystemRoot;

			/// <summary>
			/// An array of DismLanguage Structure objects representing the languages in the image.
			/// </summary>
			public IntPtr Language;

			/// <summary>
			/// The number of elements in the language array.
			/// </summary>
			public uint LanguageCount;

			/// <summary>
			/// The index number of the default language.
			/// </summary>
			public uint DefaultLanguageIndex;

			/// <summary>
			/// The customized information for the image file. A DismWimCustomizedInfo Structure type for a WIM file. NULL for a VHD image.
			/// </summary>
			public IntPtr CustomizedInfo;
		}
	}

	/// <summary>
	/// Represents the metadata of an image.
	/// </summary>
	public sealed class DismImageInfo
	{
		private readonly List<CultureInfo> languages = new List<CultureInfo>();
		private readonly DismApi.DismImageInfo_ imageInfo;
		private readonly DismWimCustomizedInfo wimCustomizedInfo;
		
		/// <summary>
		/// Creates an instance of the DismImageInfo class.
		/// </summary>
		/// <param name="imageInfoPtr">A native pointer to a DismImageInfo_ structure.</param>
		internal DismImageInfo(IntPtr imageInfoPtr)
			: this((DismApi.DismImageInfo_)Marshal.PtrToStructure(imageInfoPtr, typeof(DismApi.DismImageInfo_)))
		{
		}

		/// <summary>
		/// Creates an instance of the DismImageInfo class.
		/// </summary>
		/// <param name="imageInfo">A instance of a DismImageInfo_ structure.</param>
		internal DismImageInfo(DismApi.DismImageInfo_ imageInfo)
		{
			this.imageInfo = imageInfo;

			// Loop through each language and add it to the list
			DismApi.GetIntPtrList(this.imageInfo.Language, (int)this.imageInfo.LanguageCount, typeof(DismApi.DismLanguage)).ForEach(i => this.languages.Add(CultureInfo.GetCultureInfo(((DismApi.DismLanguage)Marshal.PtrToStructure(i, typeof(DismApi.DismLanguage))).Value)));

			// Parse the OS version from the various fields
			this.ProductVersion = new Version((int)imageInfo.MajorVersion, (int)imageInfo.MinorVersion, (int)imageInfo.Build, (int)imageInfo.SpBuild);

			// Marshal the WIM customized info
			if (this.imageInfo.CustomizedInfo != IntPtr.Zero)
			{
				this.wimCustomizedInfo = new DismWimCustomizedInfo(this.imageInfo.CustomizedInfo);
			}
		}

		/// <summary>
		/// Gets the type of the image.
		/// </summary>
		public DismImageType ImageType
		{
			get
			{
				return this.imageInfo.ImageType;
			}
		}

		/// <summary>
		/// Gets the index number, starting at 1, of the image.
		/// </summary>
		public int ImageIndex
		{
			get
			{
				return (int)this.imageInfo.ImageIndex;
			}
		}

		/// <summary>
		/// Gets the name of the image.
		/// </summary>
		public string ImageName
		{
			get
			{
				return this.imageInfo.ImageName;
			}
		}

		/// <summary>
		/// Gets the description of the image.
		/// </summary>
		public string ImageDescription
		{
			get
			{
				return this.imageInfo.ImageDescription;
			}
		}

		/// <summary>
		/// Gets the size of the image in bytes.
		/// </summary>
		public ulong ImageSize
		{
			get
			{
				return this.imageInfo.ImageSize;
			}
		}

		/// <summary>
		/// Gets the architecture of the image.
		/// </summary>
		public DismProcessorArchitecture Architecture
		{
			get
			{
				return this.imageInfo.Architecture;
			}
		}

		/// <summary>
		/// Gets the name of the product.
		/// </summary>
		public string ProductName
		{
			get
			{
				return this.imageInfo.ProductName;
			}
		}

		/// <summary>
		/// Gets thee product suite, for example, "Terminal Server".
		/// </summary>
		public string ProductSuite
		{
			get
			{
				return this.imageInfo.ProductSuite;
			}
		}

		/// <summary>
		/// Gets the product type, for example, "WinNT".
		/// </summary>
		public string ProductType
		{
			get
			{
				return this.imageInfo.ProductType;
			}
		}

		/// <summary>
		/// Gets the name of the edition.
		/// </summary>
		public string EditionId
		{
			get
			{
				return this.imageInfo.EditionId;
			}
		}

		/// <summary>
		/// Gets the type of installation such as "Client" or "Server".
		/// </summary>
		public string InstallationType
		{
			get
			{
				return this.imageInfo.InstallationType;
			}
		}

		/// <summary>
		/// Gets the hardware abstraction layer (HAL) type of the operating system.
		/// </summary>
		public string Hal
		{
			get
			{
				return this.imageInfo.Hal;
			}
		}

		/// <summary>
		/// Gets the version of the operating system contained in the image.
		/// </summary>
		public Version ProductVersion
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets a value indicating whether an image is a bootable image type.
		/// </summary>
		public DismImageBootable Bootable
		{
			get
			{
				return this.imageInfo.Bootable;
			}
		}

		/// <summary>
		/// Gets the service pack number.
		/// </summary>
		public int SpLevel
		{
			get
			{
				return (int)this.imageInfo.SpLevel;
			}
		}

		/// <summary>
		/// Gets the path to the Windows directory.
		/// </summary>
		public string SystemRoot
		{
			get
			{
				return this.imageInfo.SystemRoot;
			}
		}

		/// <summary>
		/// Gets the languages in the image.
		/// </summary>
		public IEnumerable<CultureInfo> Languages
		{
			get
			{
				return this.languages;
			}
		}

		/// <summary>
		/// Gets the default language of the image.
		/// </summary>
		public CultureInfo DefaultLanguage
		{
			get
			{
				return this.languages[this.DefaultLanguageIndex];
			}
		}

		/// <summary>
		/// The index number of the default language.
		/// </summary>
		public int DefaultLanguageIndex
		{
			get
			{
				return (int)this.imageInfo.DefaultLanguageIndex;
			}
		}

		/// <summary>
		/// The customized information for the image file. A <see cref="DismWimCustomizedInfo" /> for a WIM file. null for a VHD image.
		/// </summary>
		public DismWimCustomizedInfo CustomizedInfo
		{
			get
			{
				return this.wimCustomizedInfo;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismImageInfo"/> objects.
	/// </summary>
	public sealed class DismImageInfoCollection : DismCollection<DismImageInfo>
	{
		/// <summary>
		/// Initializes a new instance of the DismImageInfoCollection that is empty.
		/// </summary>
		internal DismImageInfoCollection()
			: base(new List<DismImageInfo>())
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismImageInfoCollection based on the specified list.
		/// </summary>
		internal DismImageInfoCollection(IList<DismImageInfo> list)
			: base(list)
		{
		}
	}
}
