﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes the metadata of a mounted image.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824755.aspx"/>
		/// <![CDATA[typedef struct _DismMountedImageInfo
		/// {
		///     PCWSTR MountPath;
		///     PCWSTR ImageFilePath;
		///     UINT ImageIndex;
		///     DismMountMode MountMode;
		///     DismMountStatus MountStatus;
		/// } DismMountedImageInfo;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismMountedImageInfo_
		{
			/// <summary>
			/// A relative or absolute path to the mounted image.
			/// </summary>
			public string MountPath;

			/// <summary>
			/// A relative or absolute path to the image file.
			/// </summary>
			public string ImageFilePath;

			/// <summary>
			/// The index number of the image. Index numbering starts at 1.
			/// </summary>
			public UInt32 ImageIndex;

			/// <summary>
			/// A <a href="DismMountMode"/> Enumeration value representing whether the image is DismReadWrite or DismReadOnly.
			/// </summary>
			public DismMountMode MountMode;

			/// <summary>
			/// A <a href="DismMountStatus"/> Enumeration value such as DismMountStatusOk.
			/// </summary>
			public DismMountStatus MountStatus;
		}
	}

	/// <summary>
	/// Represents a mounted image.
	/// </summary>
	/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824755.aspx"/>
	/// <![CDATA[typedef struct _DismMountedImageInfo
	/// {
	///     PCWSTR MountPath;
	///     PCWSTR ImageFilePath;
	///     UINT ImageIndex;
	///     DismMountMode MountMode;
	///     DismMountStatus MountStatus;
	/// } DismMountedImageInfo;]]>
	public sealed class DismMountedImageInfo
	{
		private readonly DismApi.DismMountedImageInfo_ mountedImageInfo;

		internal DismMountedImageInfo(IntPtr mountedImageInfoPtr)
			: this((DismApi.DismMountedImageInfo_)Marshal.PtrToStructure(mountedImageInfoPtr, typeof(DismApi.DismMountedImageInfo_)))
		{
		}

		internal DismMountedImageInfo(DismApi.DismMountedImageInfo_ mountedImageInfo)
		{
			this.mountedImageInfo = mountedImageInfo;
		}

		/// <summary>
		/// Gets the relative or absolute path to the mounted image.
		/// </summary>
		public string MountPath
		{
			get
			{
				return this.mountedImageInfo.MountPath;
			}
		}

		/// <summary>
		/// Gets the relative or absolute path to the image file.
		/// </summary>
		public string ImageFilePath
		{
			get
			{
				return this.mountedImageInfo.ImageFilePath;
			}
		}

		/// <summary>
		/// Gets the index number of the image. Index numbering starts at 1.
		/// </summary>
		public int ImageIndex
		{
			get
			{
				return (int)this.mountedImageInfo.ImageIndex;
			}
		}

		/// <summary>
		/// Gets the mount mode of the image.
		/// </summary>
		public DismMountMode MountMode
		{
			get
			{
				return this.mountedImageInfo.MountMode;
			}
		}

		/// <summary>
		/// Gets the status of the mounted image.
		/// </summary>
		public DismMountStatus MountStatus
		{
			get
			{
				return this.mountedImageInfo.MountStatus;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismMountedImageInfo"/> objects.
	/// </summary>
	public sealed class DismMountedImageInfoCollection : DismCollection<DismMountedImageInfo>
	{
		/// <summary>
		/// Initializes a new instance of the DismMountedImageInfoCollection class.
		/// </summary>
		internal DismMountedImageInfoCollection()
			: base(new List<DismMountedImageInfo>())
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismMountedImageInfoCollection class.
		/// </summary>
		/// <param name="list">An existing list of DismMountedImageInfo objects to expose as a read-only collection.</param>
		internal DismMountedImageInfoCollection(IList<DismMountedImageInfo> list)
			: base(list)
		{
		}
	}
}