﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes basic information about a package, including the date and time that the package was installed.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824763.aspx"/>
		/// <![CDATA[typedef struct _DismPackage
		/// {
		///     PCWSTR PackageName;
		///     DismPackageFeatureState PackageState;
		///     DismReleaseType ReleaseType;
		///     SYSTEMTIME InstallTime;
		/// } DismPackage;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismPackage_
		{
			/// <summary>
			/// The package name.
			/// </summary>
			public string PackageName;

			/// <summary>
			/// A DismPackageFeatureState Enumeration value, for example, DismStateResolved.
			/// </summary>
			public DismPackageFeatureState PackageState;

			/// <summary>
			/// A DismReleaseType Enumeration value, for example, DismReleaseTypeDriver.
			/// </summary>
			public DismReleaseType ReleaseType;

			/// <summary>
			/// The date and time that the package was installed. This field is local time relative to the servicing host computer.
			/// </summary>
			public SYSTEMTIME InstallTime;
		}
	}

	/// <summary>
	/// Represents basic information about a package, including the date and time that the package was installed.
	/// </summary>
	public sealed class DismPackage
	{
		private readonly DismApi.DismPackage_ package;

		/// <summary>
		/// Initializes a new instance of the DismPackage class.
		/// </summary>
		/// <param name="packagePtr">A pointer to a native DismPackage_ struct.</param>
		internal DismPackage(IntPtr packagePtr)
			: this((DismApi.DismPackage_)Marshal.PtrToStructure(packagePtr, typeof(DismApi.DismPackage_)))
		{

		}

		/// <summary>
		/// Initializes a new instance of the DismPackage class.
		/// </summary>
		/// <param name="package"></param>
		internal DismPackage(DismApi.DismPackage_ package)
		{
			this.package = package;
		}

		/// <summary>
		/// Gets the package name.
		/// </summary>
		public string PackageName
		{
			get
			{
				return this.package.PackageName;
			}
		}

		/// <summary>
		/// Gets the state of the package.
		/// </summary>
		public DismPackageFeatureState PackageState
		{
			get
			{
				return this.package.PackageState;
			}
		}

		/// <summary>
		/// Gets the release type of the package.
		/// </summary>
		public DismReleaseType ReleaseType
		{
			get
			{
				return this.package.ReleaseType;
			}
		}

		/// <summary>
		/// Gets the date and time the package was installed.
		/// </summary>
		public DateTime InstallTime
		{
			get
			{
				return this.package.InstallTime;
			}
		}
	}

	/// <summary>
	/// Represents a collection of <see cref="DismPackage"/> objects.
	/// </summary>
	public sealed class DismPackageCollection : DismCollection<DismPackage>
	{
		/// <summary>
		/// Initializes a new instance of the DismPackageCollection class.
		/// </summary>
		internal DismPackageCollection()
			: base(new List<DismPackage>())
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismPackageCollection class.
		/// </summary>
		/// <param name="list">An existing list of DismPackage objects to expose as a read-only collection.</param>
		internal DismPackageCollection(IList<DismPackage> list)
			: base(list)
		{
		}
	}
}