﻿using System;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes detailed package information such as the client used to install the package, the date and time that the package was installed, and support information.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824774.aspx"/>
		/// <![CDATA[typedef struct _DismPackageInfo
		/// {
		///     PCWSTR PackageName;
		///     DismPackageFeatureState PackageState;
		///     DismReleaseType ReleaseType;
		///     SYSTEMTIME InstallTime;
		///     BOOL Applicable;
		///     PCWSTR Copyright;
		///     PCWSTR Company;
		///     SYSTEMTIME CreationTime;
		///     PCWSTR DisplayName;
		///     PCWSTR Description;
		///     PCWSTR InstallClient;
		///     PCWSTR InstallPackageName;
		///     SYSTEMTIME LastUpdateTime;
		///     PCWSTR ProductName;
		///     PCWSTR ProductVersion;
		///     DismRestartType RestartRequired;
		///     DismFullyOfflineInstallableType FullyOffline;
		///     PCWSTR SupportInformation;
		///     DismCustomProperty* CustomProperty;
		///     UINT CustomPropertyCount;
		///     DismFeature* Feature;
		///     UINT FeatureCount;
		/// } DismPackageInfo; ]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismPackageInfo_
		{
			/// <summary>
			/// The name of the package.
			/// </summary>
			public string PackageName;

			/// <summary>
			/// A DismPackageFeatureState Enumeration value such as DismStateResolved.
			/// </summary>
			public DismPackageFeatureState PackageState;

			/// <summary>
			/// A DismReleaseType Enumeration value such as DismReleaseTypeUpdate.
			/// </summary>
			public DismReleaseType ReleaseType;

			/// <summary>
			/// The date and time that the package was installed. This field is local time, based on the servicing host computer.
			/// </summary>
			public SYSTEMTIME InstallTime;

			/// <summary>
			/// TRUE if the package is applicable to the image, otherwise FALSE.
			/// </summary>
			public bool Applicable;

			/// <summary>
			/// The copyright information of the package.
			/// </summary>
			public string Copyright;

			/// <summary>
			/// The company that released the package.
			/// </summary>
			public string Company;

			/// <summary>
			/// The date and time that the package was created. This field is local time, based on the time zone of the computer that created the package.
			/// </summary>
			public SYSTEMTIME CreationTime;

			/// <summary>
			/// The display name of the package.
			/// </summary>
			public string DisplayName;

			/// <summary>
			/// A description of the purpose of the package.
			/// </summary>
			public string Description;

			/// <summary>
			/// The client that installed this package.
			/// </summary>
			public string InstallClient;

			/// <summary>
			/// The original file name used for the package during installation.
			/// </summary>
			public string InstallPackageName;

			/// <summary>
			/// The date and time when this package was last updated. This field is local time, based on the servicing host computer.
			/// </summary>
			public SYSTEMTIME LastUpdateTime;

			/// <summary>
			/// The product name for this package.
			/// </summary>
			public string ProductName;

			/// <summary>
			/// The product version for this package.
			/// </summary>
			public string ProductVersion;

			/// <summary>
			/// A DismRestartType Enumeration value describing whether a restart is required after installing the package on an online image
			/// </summary>
			public DismRestartType RestartRequired;

			/// <summary>
			/// A DismFullyOfflineInstallableType Enumeration value describing whether a package can be installed offline without booting the image.
			/// </summary>
			public DismFullyOfflineInstallableType FullyOffline;

			/// <summary>
			/// A string listing additional support information for this package.
			/// </summary>
			public string SupportInformation;

			/// <summary>
			/// An array of DismCustomProperty Structure objects representing the custom properties of the package.
			/// </summary>
			public IntPtr CustomProperty;

			/// <summary>
			/// The number of elements in the CustomProperty array.
			/// </summary>
			public UInt32 CustomPropertyCount;

			/// <summary>
			/// An array of DismFeature Structure objects representing the features in the package.
			/// </summary>
			public IntPtr Feature;

			/// <summary>
			/// The number of elements in the Feature array.
			/// </summary>
			public UInt32 FeatureCount;
		}
	}

	/// <summary>
	/// Represents detailed package information such as the client used to install the package, the date and time that the package was installed, and support information.
	/// </summary>
	public sealed class DismPackageInfo
	{
		private readonly DismApi.DismPackageInfo_ packageInfo;
		private readonly DismCustomPropertyCollection customProperties = new DismCustomPropertyCollection();
		private readonly DismFeatureCollection features = new DismFeatureCollection();

		/// <summary>
		/// Initializes a new instance of the DismPackageInfo class.
		/// </summary>
		/// <param name="packageInfoPtr">A pointer to a native <see cref="DismApi.DismPackageInfo_"/> struct.</param>
		internal DismPackageInfo(IntPtr packageInfoPtr)
			: this((DismApi.DismPackageInfo_)Marshal.PtrToStructure(packageInfoPtr, typeof(DismApi.DismPackageInfo_)))
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismPackageInfo class.
		/// </summary>
		/// <param name="packageInfo">A <see cref="DismApi.DismPackageInfo_"/> struct containing data for this object.</param>
		internal DismPackageInfo(DismApi.DismPackageInfo_ packageInfo)
		{
			this.packageInfo = packageInfo;

			// See if there are any custom properties
			if (this.packageInfo.CustomPropertyCount > 0 && this.packageInfo.CustomProperty != IntPtr.Zero)
			{
				// Add each custom property to the collection
				DismApi.GetIntPtrList(this.packageInfo.CustomProperty, (int)this.packageInfo.CustomPropertyCount, typeof(DismApi.DismCustomProperty_)).ForEach(ptr => this.customProperties.Add(new DismCustomProperty(ptr)));
			}

			// See if there are any features associated with the package
			if (this.packageInfo.FeatureCount > 0 && this.packageInfo.Feature != IntPtr.Zero)
			{
				// Add each feature to the collection
				DismApi.GetIntPtrList(this.packageInfo.Feature, (int)this.packageInfo.FeatureCount, typeof(DismApi.DismFeature_)).ForEach(ptr => this.features.Add(new DismFeature(ptr)));
			}
		}

		/// <summary>
		/// Gets a value indicating if the package is applicable to the image.
		/// </summary>
		public bool Applicable
		{
			get
			{
				return this.packageInfo.Applicable;
			}
		}

		/// <summary>
		/// The copyright information of the package.
		/// </summary>
		public string Copyright
		{
			get
			{
				return this.packageInfo.Copyright;
			}
		}

		/// <summary>
		/// The company that released the package.
		/// </summary>
		public string Company
		{
			get
			{
				return this.packageInfo.Company;
			}
		}

		/// <summary>
		/// The date and time that the package was created. This field is local time, based on the time zone of the computer that created the package.
		/// </summary>
		public DateTime CreationTime
		{
			get
			{
				return this.packageInfo.CreationTime;
			}
		}

		/// <summary>
		/// The display name of the package.
		/// </summary>
		public string DisplayName
		{
			get
			{
				return this.packageInfo.DisplayName;
			}
		}

		/// <summary>
		/// A description of the purpose of the package.
		/// </summary>
		public string Description
		{
			get
			{
				return this.packageInfo.Description;
			}
		}

		/// <summary>
		/// The client that installed this package.
		/// </summary>
		public string InstallClient
		{
			get
			{
				return this.packageInfo.InstallClient;
			}
		}

		/// <summary>
		/// The original file name used for the package during installation.
		/// </summary>
		public string InstallPackageName
		{
			get
			{
				return this.packageInfo.InstallPackageName;
			}
		}

		/// <summary>
		/// The date and time when this package was last updated. This field is local time, based on the servicing host computer.
		/// </summary>
		public DateTime LastUpdateTime
		{
			get
			{
				return this.packageInfo.LastUpdateTime;
			}
		}

		/// <summary>
		/// The product name for this package.
		/// </summary>
		public string ProductName
		{
			get
			{
				return this.packageInfo.ProductName;
			}
		}

		/// <summary>
		/// The product version for this package.
		/// </summary>
		public string ProductVersion
		{
			get
			{
				return this.packageInfo.ProductVersion;
			}
		}

		/// <summary>
		/// A DismRestartType Enumeration value describing whether a restart is required after installing the package on an online image
		/// </summary>
		public DismRestartType RestartRequired
		{
			get
			{
				return this.packageInfo.RestartRequired;
			}
		}

		/// <summary>
		/// A DismFullyOfflineInstallableType Enumeration value describing whether a package can be installed offline without booting the image.
		/// </summary>
		public DismFullyOfflineInstallableType FullyOffline
		{
			get
			{
				return this.packageInfo.FullyOffline;
			}
		}

		/// <summary>
		/// A string listing additional support information for this package.
		/// </summary>
		public string SupportInformation
		{
			get
			{
				return this.packageInfo.SupportInformation;
			}
		}

		/// <summary>
		/// An array of DismCustomProperty Structure objects representing the custom properties of the package.
		/// </summary>
		public DismCustomPropertyCollection CustomProperties
		{
			get
			{
				return this.customProperties;
			}
		}

		/// <summary>
		/// An array of DismFeature Structure objects representing the features in the package.
		/// </summary>
		public DismFeatureCollection Features
		{
			get
			{
				return this.features;
			}
		}
	}
}
