﻿using System;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Describes a Windows® Imaging Format (WIM) file.
		/// </summary>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/hh824792.aspx"/>
		/// <![CDATA[typedef struct _DismWimCustomizedInfo
		/// {
		///     UINT Size;
		///     UINT DirectoryCount;
		///     UINT FileCount;
		///     SYSTEMTIME CreatedTime;
		///     SYSTEMTIME ModifiedTime;
		/// } DismWimCustomizedInfo;]]>
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode, Pack = 4)]
		internal struct DismWimCustomizedInfo_
		{
			/// <summary>
			/// The size of the DismWimCustomizedInfo structure.
			/// </summary>
			public UInt32 Size;

			/// <summary>
			/// The number of directories in the image.
			/// </summary>
			public UInt32 DirectoryCount;

			/// <summary>
			/// The number of files in the image.
			/// </summary>
			public UInt32 FileCount;

			/// <summary>
			/// The time that the image file was created.
			/// </summary>
			public SYSTEMTIME CreatedTime;

			/// <summary>
			/// The time that the image file was last modified.
			/// </summary>
			public SYSTEMTIME ModifiedTime;
		}
	}

	/// <summary>
	/// Represents a Windows® Imaging Format (WIM) file.
	/// </summary>
	public sealed class DismWimCustomizedInfo
	{
		private readonly DismApi.DismWimCustomizedInfo_ wimCustomizedInfo;

		/// <summary>
		/// Initializes a new instance of the DismWimCustomizedInfo class.
		/// </summary>
		/// <param name="wimCustomizedInfoPtr">A pointer to a native <see cref="DismApi.DismWimCustomizedInfo_"/> struct.</param>
		internal DismWimCustomizedInfo(IntPtr wimCustomizedInfoPtr)
			: this((DismApi.DismWimCustomizedInfo_)Marshal.PtrToStructure(wimCustomizedInfoPtr, typeof(DismApi.DismWimCustomizedInfo_)))
		{
		}

		/// <summary>
		/// Initializes a new instance of the DismWimCustomizedInfo class.
		/// </summary>
		/// <param name="wimCustomizedInfo">A native <see cref="DismApi.DismWimCustomizedInfo_"/> struct to copy data from.</param>
		internal DismWimCustomizedInfo(DismApi.DismWimCustomizedInfo_ wimCustomizedInfo)
		{
			this.wimCustomizedInfo = wimCustomizedInfo;
		}

		/// <summary>
		/// Gets the size of the customized info.
		/// </summary>
		public long Size
		{
			get
			{
				return this.wimCustomizedInfo.Size;
			}
		}

		/// <summary>
		/// Gets the number of directories in the image.
		/// </summary>
		public long DirectoryCount
		{
			get
			{
				return this.wimCustomizedInfo.DirectoryCount;
			}
		}

		/// <summary>
		/// Gets the number of files in the image.
		/// </summary>
		public long FileCount
		{
			get
			{
				return this.wimCustomizedInfo.FileCount;
			}
		}

		/// <summary>
		/// Gets the time that the image file was created.
		/// </summary>
		public DateTime CreatedTime
		{
			get
			{
				return this.wimCustomizedInfo.CreatedTime;
			}
		}

		/// <summary>
		/// Gets the time that the image file was last modified.
		/// </summary>
		public DateTime ModifiedTime
		{
			get
			{
				return this.wimCustomizedInfo.ModifiedTime;
			}
		}
	}
}
