﻿using System;
using System.Runtime.InteropServices;

namespace Microsoft.Dism
{
	public static partial class DismApi
	{
		/// <summary>
		/// Specifies a date and time, using individual members for the month, day, year, weekday, hour, minute, second, and millisecond. The time is either in coordinated universal time (UTC) or local time, depending on the function that is being called.
		/// </summary>
		/// <remarks>It is not recommended that you add and subtract values from the SYSTEMTIME structure to obtain relative times. Instead, you should
		/// <list type="bullet">
		///		<item><description>Convert the SYSTEMTIME structure to a FILETIME structure.</description></item>
		///		<item><description>Copy the resulting FILETIME structure to a ULARGE_INTEGER structure.</description></item>
		///		<item><description>Use normal 64-bit arithmetic on the ULARGE_INTEGER value.</description></item>
		///	</list>	
		/// The system can periodically refresh the time by synchronizing with a time source. Because the system time can be adjusted either forward or backward, do not compare system time readings to determine elapsed time. Instead, use one of the methods described in Windows Time.</remarks>
		/// <a href="http://msdn.microsoft.com/en-us/library/windows/desktop/ms724950(v=vs.85).aspx"/>
		/// <![CDATA[typedef struct _SYSTEMTIME {
		/// WORD wYear;
		///   WORD wMonth;
		///   WORD wDayOfWeek;
		///   WORD wDay;
		///   WORD wHour;
		///   WORD wMinute;
		///   WORD wSecond;
		///   WORD wMilliseconds;
		/// } SYSTEMTIME, *PSYSTEMTIME;]]>
		[StructLayout(LayoutKind.Sequential)]
		internal class SYSTEMTIME
		{
			/// <summary>
			/// The year. The valid values for this member are 1601 through 30827.
			/// </summary>
			public UInt16 wYear;

			/// <summary>
			/// The month. January = 1 and December = 12
			/// </summary>
			public UInt16 wMonth;

			/// <summary>
			/// The day of the week. Sunday = 0 and Saturday = 6
			/// </summary>
			public UInt16 wDayOfWeek;

			/// <summary>
			/// The day of the month. The valid values for this member are 1 through 31.
			/// </summary>
			public UInt16 wDay;

			/// <summary>
			/// The hour. The valid values for this member are 0 through 23.
			/// </summary>
			public UInt16 wHour;

			/// <summary>
			/// The minute. The valid values for this member are 0 through 59.
			/// </summary>
			public UInt16 wMinute;

			/// <summary>
			/// The second. The valid values for this member are 0 through 59.
			/// </summary>
			public UInt16 wSecond;

			/// <summary>
			/// The millisecond. The valid values for this member are 0 through 999.
			/// </summary>
			public UInt16 wMilliseconds;

			/// <summary>
			/// Initializes a new instance of the SYSTEMTIME class.
			/// </summary>
			/// <param name="dateTime">An existing DateTime object to copy data from.</param>
			public SYSTEMTIME(DateTime dateTime)
			{
				DateTime utc = dateTime.ToUniversalTime();

				this.wYear = (ushort)utc.Year;
				this.wMonth = (ushort)utc.Month;
				this.wDay = (ushort)utc.Day;
				this.wDayOfWeek = (ushort)utc.DayOfWeek;
				this.wHour = (ushort)utc.Hour;
				this.wMinute = (ushort)utc.Minute;
				this.wSecond = (ushort)utc.Second;
				this.wMilliseconds = (ushort)utc.Millisecond;
			}

			/// <summary>
			/// Returns the SYSTEMTIME as a <see cref="System.DateTime"/> value.
			/// </summary>
			/// <returns>A <see cref="System.DateTime"/> value.</returns>
			public DateTime ToDateTime()
			{
				if (wYear == 0)
				{
					return DateTime.MinValue;
				}
				return new DateTime(this.wYear, this.wMonth, this.wDay, this.wHour, this.wMinute, this.wSecond, DateTimeKind.Utc);
			}

			/// <summary>
			/// Converts a <see cref="System.DateTime"/> to a <see cref="SYSTEMTIME"/>.
			/// </summary>
			public static implicit operator SYSTEMTIME(DateTime dateTime)
			{
				return new SYSTEMTIME(dateTime);
			}

			/// <summary>
			/// Converts a <see cref="SYSTEMTIME"/> to a <see cref="System.DateTime"/>
			/// </summary>
			public static implicit operator DateTime(SYSTEMTIME systemTime)
			{
				return systemTime.ToDateTime();
			}
		}
	}
}
